<?php

/**
 *
 * ContentNow | Small Content Management System
 *
 * Copyright (C) 2005/2006 Andre Michaelis <kontakt@michaelis-freunde.de>
 *
 * This file may be distributed and/or modified under the terms of the
 * "GNU General Public License" version 2 as published by the Free
 * Software Foundation and appearing in the file LICENSE included in
 * the packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 *
 * The "GNU General Public License" (GPL) is available at
 * http://www.gnu.org/copyleft/gpl.html.
 *
 */

	ob_start();

	// disable magic quotes
	set_magic_quotes_runtime (0);

	// show all errors, no warnings
	error_reporting (E_ALL & ~E_NOTICE);

	// check install
	if (!file_exists('./library/configure.php')) die ('<span style="font:13px sans-serif;">Please install contentNow.<hr /><a href="./install/index.php">Install</a></span>');

	// get required classes
	if (file_exists('./require.php')) {
		require_once ('./require.php');
	} else {
		die ('<span style="font:13px sans-serif;">Fatal error loading CMS. Please contact webmaster.</span>');
	}

	// start page timer
	$timer = microtime();

	// check authentication	
	$cnSession = new cnSession();
	$cnSession->checkAuthentication();

	// make objects	
	$cnCore = new cnCore();
	$cnEngine = new cnEngine();

	// get/set language frontend
	$setCLang = (empty($_GET['clang'])) ? '' : $_GET['clang'];
	$cnCore->setContentLanguage($setCLang);
	$cnEngine->setContentLanguage($setCLang);

	// get/set language backend
	$setLang = (empty($_GET['lang'])) ? $cnCore->getBackendLanguage() : $_GET['lang'];
	$cnCore->setBackendLanguage($setLang);

	// get/set pageid
	$pid = (empty($_GET['pageid']) || $_GET['pageid'] == 1) ? $cnCore->getRootPageID() : $_GET['pageid'];
	$cnCore->setId($pid);
	$cnEngine->setId($pid);

	// set session authentication
	$errorSessionMsg = $cnSession->refreshUser($pid,$_SESSION["userid"]);

	// get mode
	$mode = (empty($_GET['mode'])) ? CN_MODE : $_GET['mode'];
	if (empty($errorSessionMsg)) {
		$cnCore->setMode($mode);
	} else {
		$cnCore->setMode('error');
	}

	// Init smarty
	$smarty = new Smarty();
	$smarty->template_dir = CN_SKINDIR;
	$smarty->compile_dir = 'smarty/templates_cn';
	$smarty->cache_dir = 'smarty/cache';
	$smarty->config_dir = 'smarty/configs';

	// Assign global var
	$smarty->assign('cnSkinDir', CN_SKINDIR );
	$smarty->assign('lang_LANG', $cnCore->lang['local_key']);
	$smarty->assign('charset', $cnCore->lang['local_charset']);
	$smarty->assign('accesKeys', $cnCore->makeAccesKeys() );

	switch ($cnCore->mode) {

		case 'content':
			// read from cnEngine
			$cnCore->setNavigationArray($cnEngine->getNavigationArray());
			$cnCore->setPageInfoArray($cnEngine->getPageInfoArray());
			$cnCore->setConfigArray($cnEngine->getConfigPage());

			// init smarty for display
			$smarty->assign('topNavigation', $cnCore->showTopNavigation() );
			$smarty->assign('navigation', $cnCore->showNavigation() );
			$smarty->assign('contentNavigation', $cnCore->showContentNav() );
			$smarty->assign('contentDisplay', $cnCore->showContent() );
			$displayObj = $smarty->fetch('_content.html');
			$smarty->assign('content', $displayObj );
		break;

		case 'meta':
			// cn actions
			if (!empty($_POST['setsavepage'])) $cnCore->setSavePage();
			if (!empty($_POST['duplicatepage'])) $cnCore->setDuplicatePage();
			if (!empty($_POST['setdeletepage'])) {
				$cnCore->setDeletePage();
				// set pageid to rootPage
				$cnCore->setId($cnCore->getRootPageID());
				$cnEngine->setId($cnCore->getRootPageID());
			}
			if (!empty($_POST['setsavenewpage'])) $cnCore->setSaveNewPage();
			if ($_GET['pageaction'] == 'moveup') $cnCore->setMovePageUp($_GET['position']);
			if ($_GET['pageaction'] == 'movedown') $cnCore->setMovePageDown($_GET['position']);

			// read from cnEngine
			$cnCore->setNavigationArray($cnEngine->getNavigationArray());
			$cnCore->setPageInfoArray($cnEngine->getPageInfoArray($_POST['deletepage']));
			$cnCore->setConfigArray($cnEngine->getConfigPage());

			// get temp obj: addpage or editpage
			$displayObj = (!empty($_POST['showaddnewpage']))
			?
			$cnCore->showMetaAddNewPage()
			:
			$cnCore->showMeta();

			// init smarty for display
			$smarty->assign('topNavigation', $cnCore->showTopNavigation() );
			$smarty->assign('navigation', $cnCore->showNavigation() );
			$smarty->assign('contentNavigation', $cnCore->showContentNav() );
			$smarty->assign('contentDisplay', $displayObj );

			$displayObj = $smarty->fetch('_meta.html');
			$smarty->assign('content', $displayObj );
		break;

		case 'config':
			// cn actions
			if (!empty($_POST['newlanguage'])) {
				if (empty($_POST['newwebsitelangid'])) {
					// reference to language array
					$errorConfigMsg = 'msg_language_install_error';
				} else {
					// reference to language array
					$errorConfigMsg = 'msg_language_install';
					$cnCore->setDuplicateContentLanguage($_POST['newwebsitelangid']);
				}
			}

			// read from cnEngine
			$cnCore->setConfigArray($cnEngine->getConfigPage());

			// init smarty for display
			$smarty->assign('topNavigation', $cnCore->showTopNavigation() );
			$smarty->assign('contentDisplay', $cnCore->showConfig($errorConfigMsg));

			$displayObj = $smarty->fetch('_config.html');
			$smarty->assign('content', $displayObj );
		break;

		case 'user':
			// cn actions
			if (!empty($_POST['savenewuser'])) $cnCore->setSaveNewUser();
			if (!empty($_GET['deluserid'])) $cnCore->setDeleteUser($_GET['deluserid']);
			$tempDisplayObj = '';
			if (!empty($_POST['newuser'])) $tempDisplayObj = $cnCore->getNewUser();
			if (!empty($_GET['userid'])) 	$tempDisplayObj = $cnCore->getInfoUser($_GET['userid']);

			// read from cnEngine
			$cnCore->setConfigArray($cnEngine->getConfigPage());

			// init smarty for display
			$smarty->assign('topNavigation', $cnCore->showTopNavigation() );
			$smarty->assign('contentDisplay', $cnCore->showUser().$tempDisplayObj );

			$displayObj = $smarty->fetch('_user.html');
			$smarty->assign('content', $displayObj );
		break;

		case 'modul' :
			// read from cnEngine
			$cnCore->setConfigArray($cnEngine->getConfigPage());

			// init smarty for display
			$smarty->assign('topNavigation', $cnCore->showTopNavigation() );
			$smarty->assign('contentDisplay', $cnCore->showModul() );

			$displayObj = $smarty->fetch('_modul.html');
			$smarty->assign('content', $displayObj );
		break;

		case 'error' :
			// override mode
			$cnCore->setMode('content');

			// read from cnEngine
			$cnCore->setContentArray($cnEngine->getActualContent());
			$cnCore->setNavigationArray($cnEngine->getNavigationArray());
			$cnCore->setPageInfoArray($cnEngine->getPageInfoArray());
			$cnCore->setConfigArray($cnEngine->getConfigPage());

			// init smarty for display error page
			$smarty->assign('topNavigation', $cnCore->showTopNavigation() );
			$smarty->assign('navigation', $cnCore->showNavigation() );
			$smarty->assign('contentNavigation', $cnCore->showContentNav() );
			$smarty->assign('contentDisplay', $errorSessionMsg );
			$displayObj = $smarty->fetch('_errormsg.html');
			$smarty->assign('content', $displayObj );
		break;
	}

	// add timer
	list($a_dec, $a_sec) = explode(" ", $timer);
	list($b_dec, $b_sec) = explode(" ", microtime());
	$timer = sprintf("%0.3f", $b_sec - $a_sec + $b_dec - $a_dec);
	$smarty->assign('timer', $timer );

	// smarty output
	$smarty->display('_page.html');

	ob_end_flush();

?>